@@ -1,9 +1,9 @@
-/* Copyright 1999-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-/* 
+/*
  * Apache example module.  Provide demonstrations of how modules do things.
  * It is not meant to be used in a production server.  Since it participates
  * in all of the processing phases, it could conceivable interfere with
@@ -24,6 +24,15 @@
  * In the interest of brevity, all functions and structures internal to
  * this module, but which may have counterparts in *real* modules, are
  * prefixed with 'x_' instead of 'example_'.
+ *
+ * IMPORTANT NOTE
+ * ==============
+ *
+ * Some of the code in this module has problems.
+ * Before using it to base your work on, see
+ *
+ * http://issues.apache.org/bugzilla/show_bug.cgi?id=29709
+ * http://issues.apache.org/bugzilla/show_bug.cgi?id=32051
  */
 
 #include "httpd.h"
@@ -449,7 +458,7 @@ static void trace_add(server_rec *s, request_rec *r, x_cfg *mconfig,
 /* at it.  If it returns any other value, it's treated as the text of an    */
 /* error message.                                                           */
 /*--------------------------------------------------------------------------*/
-/* 
+/*
  * Command handler for the NO_ARGS "Example" directive.  All we do is mark the
  * call in the trace log, and flag the applicability of the directive to the
  * current location in that location's configuration record.
@@ -480,7 +489,7 @@ static const char *cmd_example(cmd_parms *cmd, void *mconfig)
 /* calling send_http_header().  Otherwise, no header will be sent at all,   */
 /* and the output sent to the client will actually be HTTP-uncompliant.     */
 /*--------------------------------------------------------------------------*/
-/* 
+/*
  * Sample content handler.  All this does is display the call list that has
  * been built up so far.
  *
@@ -537,7 +546,7 @@ static int x_handler(request_rec *r)
     ap_rputs("  </H1>\n", r);
     ap_rputs("  <P>\n", r);
     ap_rprintf(r, "  Apache HTTP Server version: \"%s\"\n",
-	    ap_get_server_version());
+            ap_get_server_version());
     ap_rputs("  <BR>\n", r);
     ap_rprintf(r, "  Server built: \"%s\"\n", ap_get_server_built());
     ap_rputs("  </P>\n", r);;
@@ -618,7 +627,7 @@ static int x_handler(request_rec *r)
 /* see the individual handler comments below for details.                   */
 /*                                                                          */
 /*--------------------------------------------------------------------------*/
-/* 
+/*
  * This function is called during server initialisation.  Any information
  * that needs to be recorded must be in static cells, since there's no
  * configuration record.
@@ -626,7 +635,7 @@ static int x_handler(request_rec *r)
  * There is no return value.
  */
 
-/* 
+/*
  * This function is called when an heavy-weight process (such as a child) is
  * being run down or destroyed.  As with the child initialisation function,
  * any information that needs to be recorded must be in static cells, since
@@ -635,7 +644,7 @@ static int x_handler(request_rec *r)
  * There is no return value.
  */
 
-/* 
+/*
  * This function is called during server initialisation when an heavy-weight
  * process (such as a child) is being initialised.  As with the
  * module initialisation function, any information that needs to be recorded
@@ -905,7 +914,7 @@ static void x_child_init(apr_pool_t *p, server_rec *s)
  * phase.
  */
 #if 0
-static const char *x_http_method(const request_rec *r)
+static const char *x_http_scheme(const request_rec *r)
 {
     x_cfg *cfg;
 
@@ -913,8 +922,8 @@ static const char *x_http_method(const request_rec *r)
     /*
      * Log the call and exit.
      */
-    trace_add(r->server, NULL, cfg, "x_http_method()");
-    return "foo";
+    trace_add(r->server, NULL, cfg, "x_http_scheme()");
+    return "example";
 }
 
 /*
@@ -970,7 +979,7 @@ static int x_quick_handler(request_rec *r, int lookup_uri)
     /*
      * Log the call and exit.
      */
-    trace_add(r->server, NULL, cfg, "x_post_config()");
+    trace_add(r->server, NULL, cfg, "x_quick_handler()");
     return DECLINED;
 }
 
@@ -994,7 +1003,7 @@ static int x_pre_connection(conn_rec *c, void *csd)
     /*
      * Log the call and exit.
      */
-    trace_add(r->server, NULL, cfg, "x_post_config()");
+    trace_add(r->server, NULL, cfg, "x_pre_connection()");
 #endif
     return OK;
 }
@@ -1005,7 +1014,7 @@ static int x_pre_connection(conn_rec *c, void *csd)
  * some other protocol.  Both echo and POP3 modules are available as
  * examples.
  *
- * The return VALUE is OK, DECLINED, or HTTP_mumble.  If we return OK, no 
+ * The return VALUE is OK, DECLINED, or HTTP_mumble.  If we return OK, no
  * further modules are called for this phase.
  */
 static int x_process_connection(conn_rec *c)
@@ -1056,6 +1065,28 @@ static int x_translate_handler(request_rec *r)
     return DECLINED;
 }
 
+/*
+ * This routine maps r->filename to a physical file on disk.  Useful for
+ * overriding default core behavior, including skipping mapping for
+ * requests that are not file based.
+ *
+ * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
+ * further modules are called for this phase.
+ */
+static int x_map_to_storage_handler(request_rec *r)
+{
+
+    x_cfg *cfg;
+
+    cfg = our_dconfig(r);
+    /*
+     * We don't actually *do* anything here, except note the fact that we were
+     * called.
+     */
+    trace_add(r->server, r, cfg, "x_map_to_storage_handler()");
+    return DECLINED;
+}
+
 /*
  * this routine gives our module another chance to examine the request
  * headers and to take special action. This is the first phase whose
@@ -1213,9 +1244,9 @@ static int x_logger(request_rec *r)
 /* Which functions are responsible for which hooks in the server.           */
 /*                                                                          */
 /*--------------------------------------------------------------------------*/
-/* 
+/*
  * Each function our module provides to handle a particular hook is
- * specified here.  The functions are registered using 
+ * specified here.  The functions are registered using
  * ap_hook_foo(name, predecessors, successors, position)
  * where foo is the name of the hook.
  *
@@ -1231,7 +1262,7 @@ static int x_logger(request_rec *r)
  *                 modules use the same relative position, Apache will
  *                 determine which to call first.
  *                 If your module relies on another module to run first,
- *                 or another module running after yours, use the 
+ *                 or another module running after yours, use the
  *                 predecessors and/or successors.
  *
  * The number in brackets indicates the order in which the routine is called
@@ -1255,10 +1286,11 @@ static void x_register_hooks(apr_pool_t *p)
                               APR_HOOK_MIDDLE);
     ap_hook_log_transaction(x_logger, NULL, NULL, APR_HOOK_MIDDLE);
 #if 0
-    ap_hook_http_method(x_http_method, NULL, NULL, APR_HOOK_MIDDLE);
+    ap_hook_http_scheme(x_http_scheme, NULL, NULL, APR_HOOK_MIDDLE);
     ap_hook_default_port(x_default_port, NULL, NULL, APR_HOOK_MIDDLE);
 #endif
     ap_hook_translate_name(x_translate_handler, NULL, NULL, APR_HOOK_MIDDLE);
+    ap_hook_map_to_storage(x_map_to_storage_handler, NULL,NULL, APR_HOOK_MIDDLE);
     ap_hook_header_parser(x_header_parser_handler, NULL, NULL, APR_HOOK_MIDDLE);
     ap_hook_check_user_id(x_check_user_id, NULL, NULL, APR_HOOK_MIDDLE);
     ap_hook_fixups(x_fixer_upper, NULL, NULL, APR_HOOK_MIDDLE);
@@ -1277,7 +1309,7 @@ static void x_register_hooks(apr_pool_t *p)
 /* collisions of directive names between modules.                           */
 /*                                                                          */
 /*--------------------------------------------------------------------------*/
-/* 
+/*
  * List of directives specific to our module.
  */
 static const command_rec x_cmds[] =
@@ -1297,7 +1329,7 @@ static const command_rec x_cmds[] =
 /* the static hooks into our module from the other parts of the server.     */
 /*                                                                          */
 /*--------------------------------------------------------------------------*/
-/* 
+/*
  * Module definition for configuration.  If a particular callback is not
  * needed, replace its routine name below with the word NULL.
  */
