@@ -1,9 +1,9 @@
-/* Copyright 2001-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -61,7 +61,7 @@
 APR_HOOK_STRUCT(
     APR_HOOK_LINK(post_read_request)
     APR_HOOK_LINK(log_transaction)
-    APR_HOOK_LINK(http_method)
+    APR_HOOK_LINK(http_scheme)
     APR_HOOK_LINK(default_port)
 )
 
@@ -217,103 +217,113 @@ AP_DECLARE(apr_status_t) ap_rgetline_core(char **s, apr_size_t n,
     char *pos, *last_char = *s;
     int do_alloc = (*s == NULL), saw_eos = 0;
 
+    /*
+     * Initialize last_char as otherwise a random value will be compared
+     * against APR_ASCII_LF at the end of the loop if bb only contains
+     * zero-length buckets.
+     */
+    if (last_char) {
+        *last_char = '\0';
+    }
+
     for (;;) {
-    apr_brigade_cleanup(bb);
-    rv = ap_get_brigade(r->input_filters, bb, AP_MODE_GETLINE,
-                        APR_BLOCK_READ, 0);
-
-    if (rv != APR_SUCCESS) {
-        return rv;
-    }
-
-    /* Something horribly wrong happened.  Someone didn't block! */
-    if (APR_BRIGADE_EMPTY(bb)) {
-        return APR_EGENERAL;
-    }
-
-    APR_BRIGADE_FOREACH(e, bb) {
-        const char *str;
-        apr_size_t len;
-
-        /* If we see an EOS, don't bother doing anything more. */
-        if (APR_BUCKET_IS_EOS(e)) {
-            saw_eos = 1;
-            break;
-        }
-
-        rv = apr_bucket_read(e, &str, &len, APR_BLOCK_READ);
-
+        apr_brigade_cleanup(bb);
+        rv = ap_get_brigade(r->input_filters, bb, AP_MODE_GETLINE,
+                            APR_BLOCK_READ, 0);
         if (rv != APR_SUCCESS) {
             return rv;
         }
 
-        if (len == 0) {
-            /* no use attempting a zero-byte alloc (hurts when
-             * using --with-efence --enable-pool-debug) or
-             * doing any of the other logic either
-             */
-            continue;
+        /* Something horribly wrong happened.  Someone didn't block! */
+        if (APR_BRIGADE_EMPTY(bb)) {
+            return APR_EGENERAL;
         }
 
-        /* Would this overrun our buffer?  If so, we'll die. */
-        if (n < bytes_handled + len) {
-            *read = bytes_handled;
-            if (*s) {
-                /* ensure this string is NUL terminated */
-                if (bytes_handled > 0) {
-                    (*s)[bytes_handled-1] = '\0';
+        for (e = APR_BRIGADE_FIRST(bb);
+             e != APR_BRIGADE_SENTINEL(bb);
+             e = APR_BUCKET_NEXT(e))
+        {
+            const char *str;
+            apr_size_t len;
+
+            /* If we see an EOS, don't bother doing anything more. */
+            if (APR_BUCKET_IS_EOS(e)) {
+                saw_eos = 1;
+                break;
+            }
+
+            rv = apr_bucket_read(e, &str, &len, APR_BLOCK_READ);
+            if (rv != APR_SUCCESS) {
+                return rv;
+            }
+
+            if (len == 0) {
+                /* no use attempting a zero-byte alloc (hurts when
+                 * using --with-efence --enable-pool-debug) or
+                 * doing any of the other logic either
+                 */
+                continue;
+            }
+
+            /* Would this overrun our buffer?  If so, we'll die. */
+            if (n < bytes_handled + len) {
+                *read = bytes_handled;
+                if (*s) {
+                    /* ensure this string is NUL terminated */
+                    if (bytes_handled > 0) {
+                        (*s)[bytes_handled-1] = '\0';
+                    }
+                    else {
+                        (*s)[0] = '\0';
+                    }
                 }
-                else {
-                    (*s)[0] = '\0';
+                return APR_ENOSPC;
+            }
+
+            /* Do we have to handle the allocation ourselves? */
+            if (do_alloc) {
+                /* We'll assume the common case where one bucket is enough. */
+                if (!*s) {
+                    current_alloc = len;
+                    if (current_alloc < MIN_LINE_ALLOC) {
+                        current_alloc = MIN_LINE_ALLOC;
+                    }
+                    *s = apr_palloc(r->pool, current_alloc);
+                }
+                else if (bytes_handled + len > current_alloc) {
+                    /* Increase the buffer size */
+                    apr_size_t new_size = current_alloc * 2;
+                    char *new_buffer;
+
+                    if (bytes_handled + len > new_size) {
+                        new_size = (bytes_handled + len) * 2;
+                    }
+
+                    new_buffer = apr_palloc(r->pool, new_size);
+
+                    /* Copy what we already had. */
+                    memcpy(new_buffer, *s, bytes_handled);
+                    current_alloc = new_size;
+                    *s = new_buffer;
                 }
             }
-            return APR_ENOSPC;
+
+            /* Just copy the rest of the data to the end of the old buffer. */
+            pos = *s + bytes_handled;
+            memcpy(pos, str, len);
+            last_char = pos + len - 1;
+
+            /* We've now processed that new data - update accordingly. */
+            bytes_handled += len;
         }
 
-        /* Do we have to handle the allocation ourselves? */
-        if (do_alloc) {
-            /* We'll assume the common case where one bucket is enough. */
-            if (!*s) {
-                current_alloc = len;
-                if (current_alloc < MIN_LINE_ALLOC) {
-                    current_alloc = MIN_LINE_ALLOC;
-                }
-                *s = apr_palloc(r->pool, current_alloc);
-            }
-            else if (bytes_handled + len > current_alloc) {
-                /* Increase the buffer size */
-                apr_size_t new_size = current_alloc * 2;
-                char *new_buffer;
-
-                if (bytes_handled + len > new_size) {
-                    new_size = (bytes_handled + len) * 2;
-                }
-
-                new_buffer = apr_palloc(r->pool, new_size);
-
-                /* Copy what we already had. */
-                memcpy(new_buffer, *s, bytes_handled);
-                current_alloc = new_size;
-                *s = new_buffer;
-            }
-        }
-
-        /* Just copy the rest of the data to the end of the old buffer. */
-        pos = *s + bytes_handled;
-        memcpy(pos, str, len);
-        last_char = pos + len - 1;
-
-        /* We've now processed that new data - update accordingly. */
-        bytes_handled += len;
-    }
-
         /* If we got a full line of input, stop reading */
         if (last_char && (*last_char == APR_ASCII_LF)) {
             break;
         }
     }
 
-    /* Now NUL-terminate the string at the end of the line; 
+    /* Now NUL-terminate the string at the end of the line;
      * if the last-but-one character is a CR, terminate there */
     if (last_char > *s && last_char[-1] == APR_ASCII_CR) {
         last_char--;
@@ -327,52 +337,50 @@ AP_DECLARE(apr_status_t) ap_rgetline_core(char **s, apr_size_t n,
      */
     if (fold && bytes_handled && !saw_eos) {
         for (;;) {
-        const char *str;
-        apr_size_t len;
-        char c;
+            const char *str;
+            apr_size_t len;
+            char c;
 
-        /* Clear the temp brigade for this filter read. */
-        apr_brigade_cleanup(bb);
+            /* Clear the temp brigade for this filter read. */
+            apr_brigade_cleanup(bb);
 
-        /* We only care about the first byte. */
-        rv = ap_get_brigade(r->input_filters, bb, AP_MODE_SPECULATIVE,
-                            APR_BLOCK_READ, 1);
+            /* We only care about the first byte. */
+            rv = ap_get_brigade(r->input_filters, bb, AP_MODE_SPECULATIVE,
+                                APR_BLOCK_READ, 1);
+            if (rv != APR_SUCCESS) {
+                return rv;
+            }
 
-        if (rv != APR_SUCCESS) {
-            return rv;
-        }
-
-        if (APR_BRIGADE_EMPTY(bb)) {
+            if (APR_BRIGADE_EMPTY(bb)) {
                 break;
-        }
+            }
 
-        e = APR_BRIGADE_FIRST(bb);
+            e = APR_BRIGADE_FIRST(bb);
 
-        /* If we see an EOS, don't bother doing anything more. */
-        if (APR_BUCKET_IS_EOS(e)) {
+            /* If we see an EOS, don't bother doing anything more. */
+            if (APR_BUCKET_IS_EOS(e)) {
                 break;
-        }
+            }
 
-        rv = apr_bucket_read(e, &str, &len, APR_BLOCK_READ);
-
-        if (rv != APR_SUCCESS) {
+            rv = apr_bucket_read(e, &str, &len, APR_BLOCK_READ);
+            if (rv != APR_SUCCESS) {
                 apr_brigade_cleanup(bb);
-            return rv;
-        }
+                return rv;
+            }
 
-        /* Found one, so call ourselves again to get the next line.
-         *
-         * FIXME: If the folding line is completely blank, should we
-         * stop folding?  Does that require also looking at the next
-         * char?
-         */
+            /* Found one, so call ourselves again to get the next line.
+             *
+             * FIXME: If the folding line is completely blank, should we
+             * stop folding?  Does that require also looking at the next
+             * char?
+             */
             /* When we call destroy, the buckets are deleted, so save that
              * one character we need.  This simplifies our execution paths
              * at the cost of one character read.
              */
             c = *str;
-        if (c == APR_ASCII_BLANK || c == APR_ASCII_TAB) {
-            /* Do we have enough space? We may be full now. */
+            if (c == APR_ASCII_BLANK || c == APR_ASCII_TAB) {
+                /* Do we have enough space? We may be full now. */
                 if (bytes_handled >= n) {
                     *read = n;
                     /* ensure this string is terminated */
@@ -380,45 +388,45 @@ AP_DECLARE(apr_status_t) ap_rgetline_core(char **s, apr_size_t n,
                     return APR_ENOSPC;
                 }
                 else {
-                apr_size_t next_size, next_len;
-                char *tmp;
+                    apr_size_t next_size, next_len;
+                    char *tmp;
 
-                /* If we're doing the allocations for them, we have to
-                 * give ourselves a NULL and copy it on return.
-                 */
-                if (do_alloc) {
-                    tmp = NULL;
-                } else {
-                    /* We're null terminated. */
-                    tmp = last_char;
-                }
+                    /* If we're doing the allocations for them, we have to
+                     * give ourselves a NULL and copy it on return.
+                     */
+                    if (do_alloc) {
+                        tmp = NULL;
+                    } else {
+                        /* We're null terminated. */
+                        tmp = last_char;
+                    }
 
-                next_size = n - bytes_handled;
+                    next_size = n - bytes_handled;
 
                     rv = ap_rgetline_core(&tmp, next_size,
                                           &next_len, r, 0, bb);
+                    if (rv != APR_SUCCESS) {
+                        return rv;
+                    }
 
-                if (rv != APR_SUCCESS) {
-                    return rv;
-                }
+                    if (do_alloc && next_len > 0) {
+                        char *new_buffer;
+                        apr_size_t new_size = bytes_handled + next_len + 1;
 
-                if (do_alloc && next_len > 0) {
-                    char *new_buffer;
-                    apr_size_t new_size = bytes_handled + next_len + 1;
+                        /* we need to alloc an extra byte for a null */
+                        new_buffer = apr_palloc(r->pool, new_size);
 
-                    /* we need to alloc an extra byte for a null */
-                    new_buffer = apr_palloc(r->pool, new_size);
+                        /* Copy what we already had. */
+                        memcpy(new_buffer, *s, bytes_handled);
 
-                    /* Copy what we already had. */
-                    memcpy(new_buffer, *s, bytes_handled);
-
-                    /* copy the new line, including the trailing null */
-                    memcpy(new_buffer + bytes_handled, tmp, next_len + 1);
-                    *s = new_buffer;
-                }
+                        /* copy the new line, including the trailing null */
+                        memcpy(new_buffer + bytes_handled, tmp, next_len + 1);
+                        *s = new_buffer;
+                    }
 
+                    last_char += next_len;
                     bytes_handled += next_len;
-            }
+                }
             }
             else { /* next character is not tab or space */
                 break;
@@ -489,6 +497,16 @@ AP_CORE_DECLARE(void) ap_parse_uri(request_rec *r, const char *uri)
 
     r->unparsed_uri = apr_pstrdup(r->pool, uri);
 
+    /* http://issues.apache.org/bugzilla/show_bug.cgi?id=31875
+     * http://issues.apache.org/bugzilla/show_bug.cgi?id=28450
+     *
+     * This is not in fact a URI, it's a path.  That matters in the
+     * case of a leading double-slash.  We need to resolve the issue
+     * by normalising that out before treating it as a URI.
+     */
+    while ((uri[0] == '/') && (uri[1] == '/')) {
+        ++uri ;
+    }
     if (r->method_number == M_CONNECT) {
         status = apr_uri_parse_hostinfo(r->pool, uri, &r->parsed_uri);
     }
@@ -502,7 +520,7 @@ AP_CORE_DECLARE(void) ap_parse_uri(request_rec *r, const char *uri)
     if (status == APR_SUCCESS) {
         /* if it has a scheme we may need to do absoluteURI vhost stuff */
         if (r->parsed_uri.scheme
-            && !strcasecmp(r->parsed_uri.scheme, ap_http_method(r))) {
+            && !strcasecmp(r->parsed_uri.scheme, ap_http_scheme(r))) {
             r->hostname = r->parsed_uri.hostname;
         }
         else if (r->method_number == M_CONNECT) {
@@ -590,7 +608,6 @@ static int read_request_line(request_rec *r, apr_bucket_brigade *bb)
                 r->proto_num = HTTP_VERSION(1,0);
                 r->protocol  = apr_pstrdup(r->pool, "HTTP/1.0");
             }
-
             return 0;
         }
     } while ((len <= 0) && (++num_blank_lines < max_blank_lines));
@@ -673,26 +690,24 @@ AP_DECLARE(void) ap_get_mime_headers_core(request_rec *r, apr_bucket_brigade *bb
         rv = ap_rgetline(&field, r->server->limit_req_fieldsize + 2,
                          &len, r, 0, bb);
 
-        /* ap_rgetline returns APR_ENOSPC if it fills up the buffer before
-         * finding the end-of-line.  This is only going to happen if it
-         * exceeds the configured limit for a field size.
-         */
-        if (rv == APR_ENOSPC && field) {
-            r->status = HTTP_BAD_REQUEST;
-            /* insure ap_escape_html will terminate correctly */
-            field[len - 1] = '\0';
-            apr_table_setn(r->notes, "error-notes",
-                           apr_pstrcat(r->pool,
-                                       "Size of a request header field "
-                                       "exceeds server limit.<br />\n"
-                                       "<pre>\n",
-                                       ap_escape_html(r->pool, field),
-                                       "</pre>\n", NULL));
-            return;
-        }
-
         if (rv != APR_SUCCESS) {
             r->status = HTTP_BAD_REQUEST;
+
+            /* ap_rgetline returns APR_ENOSPC if it fills up the buffer before
+             * finding the end-of-line.  This is only going to happen if it
+             * exceeds the configured limit for a field size.
+             */
+            if (rv == APR_ENOSPC && field) {
+                /* insure ap_escape_html will terminate correctly */
+                field[len - 1] = '\0';
+                apr_table_setn(r->notes, "error-notes",
+                               apr_pstrcat(r->pool,
+                                           "Size of a request header field "
+                                           "exceeds server limit.<br />\n"
+                                           "<pre>\n",
+                                           ap_escape_html(r->pool, field),
+                                           "</pre>\n", NULL));
+            }
             return;
         }
 
@@ -706,14 +721,14 @@ AP_DECLARE(void) ap_get_mime_headers_core(request_rec *r, apr_bucket_brigade *bb
                  */
                 apr_size_t fold_len = last_len + len + 1; /* trailing null */
 
-                if ((fold_len - 1) > r->server->limit_req_fieldsize) {
+                if (fold_len >= (apr_size_t)(r->server->limit_req_fieldsize)) {
                     r->status = HTTP_BAD_REQUEST;
                     /* report what we have accumulated so far before the
                      * overflow (last_field) as the field with the problem
                      */
                     apr_table_setn(r->notes, "error-notes",
                                    apr_pstrcat(r->pool,
-                                               "Size of a request header field " 
+                                               "Size of a request header field "
                                                "after folding "
                                                "exceeds server limit.<br />\n"
                                                "<pre>\n",
@@ -759,7 +774,7 @@ AP_DECLARE(void) ap_get_mime_headers_core(request_rec *r, apr_bucket_brigade *bb
                                                "</pre>\n", NULL));
                     return;
                 }
-                
+
                 tmp_field = value - 1; /* last character of field-name */
 
                 *value++ = '\0'; /* NUL-terminate at colon */
@@ -769,11 +784,11 @@ AP_DECLARE(void) ap_get_mime_headers_core(request_rec *r, apr_bucket_brigade *bb
                 }
 
                 /* Strip LWS after field-name: */
-                while (tmp_field > last_field 
+                while (tmp_field > last_field
                        && (*tmp_field == ' ' || *tmp_field == '\t')) {
                     *tmp_field-- = '\0';
                 }
-                
+
                 /* Strip LWS after field-value: */
                 tmp_field = last_field + last_len - 1;
                 while (tmp_field > value
@@ -782,7 +797,7 @@ AP_DECLARE(void) ap_get_mime_headers_core(request_rec *r, apr_bucket_brigade *bb
                 }
 
                 apr_table_addn(r->headers_in, last_field, value);
-                
+
                 /* reset the alloc_len so that we'll allocate a new
                  * buffer if we have to do any more folding: we can't
                  * use the previous buffer because its contents are
@@ -826,6 +841,8 @@ request_rec *ap_read_request(conn_rec *conn)
     const char *expect;
     int access_status;
     apr_bucket_brigade *tmp_bb;
+    apr_socket_t *csd;
+    apr_interval_time_t cur_timeout;
 
     apr_pool_create(&p, conn->pool);
     apr_pool_tag(p, "request");
@@ -863,6 +880,11 @@ request_rec *ap_read_request(conn_rec *conn)
     r->status          = HTTP_REQUEST_TIME_OUT;  /* Until we get a request */
     r->the_request     = NULL;
 
+    /* Begin by presuming any module can make its own path_info assumptions,
+     * until some module interjects and changes the value.
+     */
+    r->used_path_info = AP_REQ_DEFAULT_PATH_INFO;
+
     tmp_bb = apr_brigade_create(r->pool, r->connection->bucket_alloc);
 
     /* Get the request... */
@@ -881,6 +903,17 @@ request_rec *ap_read_request(conn_rec *conn)
         return NULL;
     }
 
+    /* We may have been in keep_alive_timeout mode, so toggle back
+     * to the normal timeout mode as we fetch the header lines,
+     * as necessary.
+     */
+    csd = ap_get_module_config(conn->conn_config, &core_module);
+    apr_socket_timeout_get(csd, &cur_timeout);
+    if (cur_timeout != conn->base_server->timeout) {
+        apr_socket_timeout_set(csd, conn->base_server->timeout);
+        cur_timeout = conn->base_server->timeout;
+    }
+
     if (!r->assbackwards) {
         ap_get_mime_headers_core(r, tmp_bb);
         if (r->status != HTTP_REQUEST_TIME_OUT) {
@@ -931,6 +964,14 @@ request_rec *ap_read_request(conn_rec *conn)
      */
     ap_update_vhost_from_headers(r);
 
+    /* Toggle to the Host:-based vhost's timeout mode to fetch the
+     * request body and send the response body, if needed.
+     */
+    if (cur_timeout != r->server->timeout) {
+        apr_socket_timeout_set(csd, r->server->timeout);
+        cur_timeout = r->server->timeout;
+    }
+
     /* we may have switched to another server */
     r->per_dir_config = r->server->lookup_defaults;
 
@@ -950,6 +991,16 @@ request_rec *ap_read_request(conn_rec *conn)
                       "(see RFC2616 section 14.23): %s", r->uri);
     }
 
+    /*
+     * Add the HTTP_IN filter here to ensure that ap_discard_request_body
+     * called by ap_die and by ap_send_error_response works correctly on
+     * status codes that do not cause the connection to be dropped and
+     * in situations where the connection should be kept alive.
+     */
+
+    ap_add_input_filter_handle(ap_http_input_filter_handle,
+                               NULL, r, r->connection);
+
     if (r->status != HTTP_OK) {
         ap_send_error_response(r, 0);
         ap_update_child_status(conn->sbh, SERVER_BUSY_LOG, r);
@@ -957,6 +1008,13 @@ request_rec *ap_read_request(conn_rec *conn)
         return r;
     }
 
+    if ((access_status = ap_run_post_read_request(r))) {
+        ap_die(access_status, r);
+        ap_update_child_status(conn->sbh, SERVER_BUSY_LOG, r);
+        ap_run_log_transaction(r);
+        return NULL;
+    }
+
     if (((expect = apr_table_get(r->headers_in, "Expect")) != NULL)
         && (expect[0] != '\0')) {
         /*
@@ -980,19 +1038,30 @@ request_rec *ap_read_request(conn_rec *conn)
         }
     }
 
-    ap_add_input_filter_handle(ap_http_input_filter_handle,
-                               NULL, r, r->connection);
-
-    if ((access_status = ap_run_post_read_request(r))) {
-        ap_die(access_status, r);
-        ap_update_child_status(conn->sbh, SERVER_BUSY_LOG, r);
-        ap_run_log_transaction(r);
-        return NULL;
-    }
-
     return r;
 }
 
+/* if a request with a body creates a subrequest, clone the original request's
+ * input headers minus any headers pertaining to the body which has already
+ * been read.  out-of-line helper function for ap_set_sub_req_protocol.
+ */
+
+static void clone_headers_no_body(request_rec *rnew,
+                                  const request_rec *r)
+{
+    rnew->headers_in = apr_table_copy(rnew->pool, r->headers_in);
+    apr_table_unset(rnew->headers_in, "Content-Encoding");
+    apr_table_unset(rnew->headers_in, "Content-Language");
+    apr_table_unset(rnew->headers_in, "Content-Length");
+    apr_table_unset(rnew->headers_in, "Content-Location");
+    apr_table_unset(rnew->headers_in, "Content-MD5");
+    apr_table_unset(rnew->headers_in, "Content-Range");
+    apr_table_unset(rnew->headers_in, "Content-Type");
+    apr_table_unset(rnew->headers_in, "Expires");
+    apr_table_unset(rnew->headers_in, "Last-Modified");
+    apr_table_unset(rnew->headers_in, "Transfer-Encoding");
+}
+
 /*
  * A couple of other functions which initialize some of the fields of
  * a request structure, as appropriate for adjuncts of one kind or another
@@ -1014,7 +1083,16 @@ AP_DECLARE(void) ap_set_sub_req_protocol(request_rec *rnew,
 
     rnew->status          = HTTP_OK;
 
-    rnew->headers_in      = r->headers_in;
+    /* did the original request have a body?  (e.g. POST w/SSI tags)
+     * if so, make sure the subrequest doesn't inherit body headers
+     */
+    if (apr_table_get(r->headers_in, "Content-Length")
+        || apr_table_get(r->headers_in, "Transfer-Encoding")) {
+        clone_headers_no_body(rnew, r);
+    } else {
+        /* no body (common case).  clone headers the cheap way */
+        rnew->headers_in      = r->headers_in;
+    }
     rnew->subprocess_env  = apr_table_copy(rnew->pool, r->subprocess_env);
     rnew->headers_out     = apr_table_make(rnew->pool, 5);
     rnew->err_headers_out = apr_table_make(rnew->pool, 5);
@@ -1102,9 +1180,9 @@ AP_DECLARE(void) ap_note_digest_auth_failure(request_rec *r)
     apr_table_setn(r->err_headers_out,
                    (PROXYREQ_PROXY == r->proxyreq) ? "Proxy-Authenticate"
                                                    : "WWW-Authenticate",
-                   /* need APR_TIME_T_FMT_HEX */
-                   apr_psprintf(r->pool, "Digest realm=\"%s\", nonce=\"%llx\"",
-                                ap_auth_name(r), r->request_time));
+                   apr_psprintf(r->pool, "Digest realm=\"%s\", nonce=\""
+                                "%" APR_UINT64_T_HEX_FMT "\"",
+                                ap_auth_name(r), (apr_uint64_t)r->request_time));
 }
 
 AP_DECLARE(int) ap_get_basic_auth_pw(request_rec *r, const char **pw)
@@ -1240,7 +1318,19 @@ AP_CORE_DECLARE_NONSTD(apr_status_t) ap_content_length_filter(
      * We can only set a C-L in the response header if we haven't already
      * sent any buckets on to the next output filter for this request.
      */
-    if (ctx->data_sent == 0 && eos) {
+    if (ctx->data_sent == 0 && eos &&
+        /* don't whack the C-L if it has already been set for a HEAD
+         * by something like proxy.  the brigade only has an EOS bucket
+         * in this case, making r->bytes_sent zero.
+         *
+         * if r->bytes_sent > 0 we have a (temporary) body whose length may 
+         * have been changed by a filter.  the C-L header might not have been 
+         * updated so we do it here.  long term it would be cleaner to have 
+         * such filters update or remove the C-L header, and just use it 
+         * if present.
+         */
+        !(r->header_only && r->bytes_sent == 0 &&   
+            apr_table_get(r->headers_out, "Content-Length"))) {
         ap_set_content_length(r, r->bytes_sent);
     }
 
@@ -1547,7 +1637,7 @@ AP_IMPLEMENT_HOOK_RUN_ALL(int,post_read_request,
                           (request_rec *r), (r), OK, DECLINED)
 AP_IMPLEMENT_HOOK_RUN_ALL(int,log_transaction,
                           (request_rec *r), (r), OK, DECLINED)
-AP_IMPLEMENT_HOOK_RUN_FIRST(const char *,http_method,
+AP_IMPLEMENT_HOOK_RUN_FIRST(const char *,http_scheme,
                             (const request_rec *r), (r), NULL)
 AP_IMPLEMENT_HOOK_RUN_FIRST(unsigned short,default_port,
                             (const request_rec *r), (r), 0)
