@@ -255,9 +255,6 @@ APR_DECLARE(apr_status_t) apr_filepath_root(const char **rootpath,
             }
             else {
                 newpath = apr_pstrndup(p, testpath, delim1 - testpath);
-                if (flags & APR_FILEPATH_TRUENAME) {
-                    newpath[delim1 - testpath] = seperator[0];
-                }
             }
             if (flags & APR_FILEPATH_TRUENAME) {
                 newpath[0] = seperator[0];
@@ -803,16 +800,16 @@ APR_DECLARE(apr_status_t) apr_filepath_merge(char **newpath,
      * is still within given basepath.  Note that the root path 
      * segment is thoroughly tested prior to path parsing.
      */
-    if (flags & APR_FILEPATH_NOTABOVEROOT) {
-        if (memcmp(basepath, path + rootlen, baselen))
+    if ((flags & APR_FILEPATH_NOTABOVEROOT) && baselen) {
+        if (memcmp(basepath, path + rootlen, baselen) != 0)
             return APR_EABOVEROOT;
-
-        /* Ahem... if we weren't given a trailing slash on the basepath,
-         * we better be sure that /foo wasn't replaced with /foobar!
-         */
+ 
+         /* Ahem... if we have a basepath without a trailing slash,
+          * we better be sure that /foo wasn't replaced with /foobar!
+          */
         if (basepath[baselen - 1] != '/' && basepath[baselen - 1] != '\\'
-                && path[rootlen + baselen] && path[rootlen + baselen] != '/' 
-                                           && path[rootlen + baselen] != '\\')
+              && path[rootlen + baselen] && path[rootlen + baselen] != '/' 
+                                         && path[rootlen + baselen] != '\\')
             return APR_EABOVEROOT;
     }
 
@@ -841,8 +838,8 @@ APR_DECLARE(apr_status_t) apr_filepath_merge(char **newpath,
             }
             /* Null term for stat! */
             path[keptlen + seglen] = '\0';
-            if ((rv = apr_lstat(&finfo, path, 
-                                APR_FINFO_TYPE | APR_FINFO_NAME, p))
+            if ((rv = apr_stat(&finfo, path, 
+                               APR_FINFO_LINK | APR_FINFO_TYPE | APR_FINFO_NAME, p))
                 == APR_SUCCESS) {
                 apr_size_t namelen = strlen(finfo.name);
 
