@@ -155,7 +155,7 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
  */
 #define APR_OS_START_SYSERR    (APR_OS_START_EAIERR + APR_OS_ERRSPACE_SIZE)
 
-/** no error. @see APR_STATUS_IS_SUCCESS */
+/** no error. */
 #define APR_SUCCESS 0
 
 /** 
@@ -269,6 +269,8 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 #define APR_ESYMNOTFOUND   (APR_OS_START_ERROR + 26)
 /** @see APR_STATUS_IS_EPROC_UNKNOWN */
 #define APR_EPROC_UNKNOWN  (APR_OS_START_ERROR + 27)
+/** @see APR_STATUS_IS_ENOTENOUGHENTROPY */
+#define APR_ENOTENOUGHENTROPY (APR_OS_START_ERROR + 28)
 /** @} */
 
 /** 
@@ -276,7 +278,7 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
  * @warning For any particular error condition, more than one of these tests
  *      may match. This is because platform-specific error codes may not
  *      always match the semantics of the POSIX codes these tests (and the
- *      correcponding APR error codes) are named after. A notable example
+ *      corresponding APR error codes) are named after. A notable example
  *      are the APR_STATUS_IS_ENOENT and APR_STATUS_IS_ENOTDIR tests on
  *      Win32 platforms. The programmer should always be aware of this and
  *      adjust the order of the tests accordingly.
@@ -357,6 +359,9 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 /** The given process was not recognized by APR. */
 #define APR_STATUS_IS_EPROC_UNKNOWN(s)  ((s) == APR_EPROC_UNKNOWN)
 
+/** APR could not gather enough entropy to continue. */
+#define APR_STATUS_IS_ENOTENOUGHENTROPY(s) ((s) == APR_ENOTENOUGHENTROPY)
+
 /** @} */
 
 /** 
@@ -462,8 +467,8 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
  */
 #define APR_STATUS_IS_TIMEUP(s)         ((s) == APR_TIMEUP)
 /** 
- * The character conversion stopped because of an incomplete character or 
- * shift sequence at the end  of the input buffer.
+ * The operation was incomplete although some processing was performed
+ * and the results are partially valid.
  * @warning
  * always use this test, as platform-specific variances may meet this
  * more than one error code 
@@ -705,7 +710,8 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 #define APR_ECONNRESET     (APR_OS_START_CANONERR + 19)
 #endif
 
-/** @see APR_STATUS_IS_ETIMEDOUT */
+/** @see APR_STATUS_IS_ETIMEDOUT 
+ *  @deprecated */
 #ifdef ETIMEDOUT
 #define APR_ETIMEDOUT ETIMEDOUT
 #else
@@ -782,9 +788,6 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
  */
 #define APR_OS2_STATUS(e) (APR_FROM_OS_ERROR(e))
 
-#define APR_STATUS_IS_SUCCESS(s)           ((s) == APR_SUCCESS \
-                || (s) == APR_OS_START_SYSERR + NO_ERROR)
-
 /* These can't sit in a private header, so in spite of the extra size, 
  * they need to be made available here.
  */
@@ -880,7 +883,11 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
                 || (s) == APR_OS_START_SYSERR + SOCECONNABORTED)
 #define APR_STATUS_IS_ECONNRESET(s)     ((s) == APR_ECONNRESET \
                 || (s) == APR_OS_START_SYSERR + SOCECONNRESET)
-#define APR_STATUS_IS_ETIMEDOUT(s)      ((s) == APR_ETIMEDOUT \
+/* XXX deprecated */
+#define APR_STATUS_IS_ETIMEDOUT(s)         ((s) == APR_ETIMEDOUT \
+                || (s) == APR_OS_START_SYSERR + SOCETIMEDOUT)    
+#undef APR_STATUS_IS_TIMEUP
+#define APR_STATUS_IS_TIMEUP(s)         ((s) == APR_TIMEUP \
                 || (s) == APR_OS_START_SYSERR + SOCETIMEDOUT)    
 #define APR_STATUS_IS_EHOSTUNREACH(s)   ((s) == APR_EHOSTUNREACH \
                 || (s) == APR_OS_START_SYSERR + SOCEHOSTUNREACH)
@@ -939,9 +946,6 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 #define apr_get_netos_error()   (APR_FROM_OS_ERROR(WSAGetLastError()))
 #define apr_set_netos_error(e)   (WSASetLastError(APR_TO_OS_ERROR(e)))
 
-#define APR_STATUS_IS_SUCCESS(s)           ((s) == APR_SUCCESS \
-                || (s) == APR_OS_START_SYSERR + ERROR_SUCCESS)
-
 /* APR CANONICAL ERROR TESTS */
 #define APR_STATUS_IS_EACCES(s)         ((s) == APR_EACCES \
                 || (s) == APR_OS_START_SYSERR + ERROR_ACCESS_DENIED \
@@ -1015,7 +1019,12 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 #define APR_STATUS_IS_ECONNRESET(s)     ((s) == APR_ECONNRESET \
                 || (s) == APR_OS_START_SYSERR + ERROR_NETNAME_DELETED \
                 || (s) == APR_OS_START_SYSERR + WSAECONNRESET)
-#define APR_STATUS_IS_ETIMEDOUT(s)      ((s) == APR_ETIMEDOUT \
+/* XXX deprecated */
+#define APR_STATUS_IS_ETIMEDOUT(s)         ((s) == APR_ETIMEDOUT \
+                || (s) == APR_OS_START_SYSERR + WSAETIMEDOUT \
+                || (s) == APR_OS_START_SYSERR + WAIT_TIMEOUT)
+#undef APR_STATUS_IS_TIMEUP
+#define APR_STATUS_IS_TIMEUP(s)         ((s) == APR_TIMEUP \
                 || (s) == APR_OS_START_SYSERR + WSAETIMEDOUT \
                 || (s) == APR_OS_START_SYSERR + WAIT_TIMEOUT)
 #define APR_STATUS_IS_EHOSTUNREACH(s)   ((s) == APR_EHOSTUNREACH \
@@ -1037,7 +1046,7 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 #define APR_STATUS_IS_ENOTEMPTY(s)      ((s) == APR_ENOTEMPTY \
                 || (s) == APR_OS_START_SYSERR + ERROR_DIR_NOT_EMPTY)
 
-#elif defined(NETWARE) && !defined(DOXYGEN) /* !defined(OS2) && !defined(WIN32) */
+#elif defined(NETWARE) && defined(USE_WINSOCK) && !defined(DOXYGEN) /* !defined(OS2) && !defined(WIN32) */
 
 #define APR_FROM_OS_ERROR(e) (e == 0 ? APR_SUCCESS : e + APR_OS_START_SYSERR)
 #define APR_TO_OS_ERROR(e)   (e == 0 ? APR_SUCCESS : e - APR_OS_START_SYSERR)
@@ -1049,8 +1058,6 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 #define apr_get_netos_error()   (APR_FROM_OS_ERROR(WSAGetLastError()))
 #define apr_set_netos_error(e)  (WSASetLastError(APR_TO_OS_ERROR(e)))
 
-#define APR_STATUS_IS_SUCCESS(s)           ((s) == APR_SUCCESS)
-
 /* APR CANONICAL ERROR TESTS */
 #define APR_STATUS_IS_EACCES(s)         ((s) == APR_EACCES)
 #define APR_STATUS_IS_EEXIST(s)         ((s) == APR_EEXIST)
@@ -1080,7 +1087,12 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
                 || (s) == APR_OS_START_SYSERR + WSAECONNABORTED)
 #define APR_STATUS_IS_ECONNRESET(s)     ((s) == APR_ECONNRESET \
                 || (s) == APR_OS_START_SYSERR + WSAECONNRESET)
-#define APR_STATUS_IS_ETIMEDOUT(s)      ((s) == APR_ETIMEDOUT \
+/* XXX deprecated */
+#define APR_STATUS_IS_ETIMEDOUT(s)       ((s) == APR_ETIMEDOUT \
+                || (s) == APR_OS_START_SYSERR + WSAETIMEDOUT \
+                || (s) == APR_OS_START_SYSERR + WAIT_TIMEOUT)
+#undef APR_STATUS_IS_TIMEUP
+#define APR_STATUS_IS_TIMEUP(s)         ((s) == APR_TIMEUP \
                 || (s) == APR_OS_START_SYSERR + WSAETIMEDOUT \
                 || (s) == APR_OS_START_SYSERR + WAIT_TIMEOUT)
 #define APR_STATUS_IS_EHOSTUNREACH(s)   ((s) == APR_EHOSTUNREACH \
@@ -1108,14 +1120,11 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
  */
 #define apr_get_netos_error() (errno)
 #define apr_set_netos_error(e) (errno = (e))
-/** @} */
 
 /** 
  * @addtogroup APR_STATUS_IS
  * @{
  */
-/** no error */
-#define APR_STATUS_IS_SUCCESS(s)        ((s) == APR_SUCCESS)
 
 /** permission denied */
 #define APR_STATUS_IS_EACCES(s)         ((s) == APR_EACCES)
@@ -1138,7 +1147,12 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 /** not a directory */
 #define APR_STATUS_IS_ENOTDIR(s)        ((s) == APR_ENOTDIR)
 /** no space left on device */
+#ifdef EDQUOT
+#define APR_STATUS_IS_ENOSPC(s)         ((s) == APR_ENOSPC \
+                                      || (s) == EDQUOT)
+#else
 #define APR_STATUS_IS_ENOSPC(s)         ((s) == APR_ENOSPC)
+#endif
 /** not enough memory */
 #define APR_STATUS_IS_ENOMEM(s)         ((s) == APR_ENOMEM)
 /** too many open files */
@@ -1191,8 +1205,9 @@ APR_DECLARE(char *) apr_strerror(apr_status_t statcode, char *buf,
 
 /** Connection Reset by peer */
 #define APR_STATUS_IS_ECONNRESET(s)      ((s) == APR_ECONNRESET)
-/** Operation timed out */
-#define APR_STATUS_IS_ETIMEDOUT(s)       ((s) == APR_ETIMEDOUT)    
+/** Operation timed out
+ *  @deprecated */
+#define APR_STATUS_IS_ETIMEDOUT(s)      ((s) == APR_ETIMEDOUT)
 /** no route to host */
 #define APR_STATUS_IS_EHOSTUNREACH(s)    ((s) == APR_EHOSTUNREACH)
 /** network is unreachable */
